/*
<trigger>
	<usage>bsf ecp_update_duplicate_status.js</usage>
	<version>5.0</version>
	<description>
		根据当前 CR 的状态转变重复的 CR。重复的 CR 是使用“重复”关系与当前 CR 相关的 CR。
		目标状态是初始 CR 状态加上“重复”前缀。例如，“分析中”变为“重复分析中”。
		如果重复项的状态为“重复已评估”或“重复已做出结论”，那么不执行转变。
	</description>
</trigger>
*/

// Import packages
importPackage(Packages.com.telelogic.cs.api);

// Script specific global variables
const script_name = 'ecp_update_duplicate_status.js';
const script_version = '20080604';
const script_usage = 'ecp_update_duplicate_status.js';
const script_example = 'ecp_update_duplicate_status.js';
const log_header = 'Script bsf ' + script_name + ': ';
const script_debug = false;
const script_dcm_modifiable_check = true;

// Get objects
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');
var options = OptionParser.parse(args);

if (script_debug) 
{
	log.info(log_header);
	log.info('    Version: ' + script_version);
	log.info('    Arguments: ' + args.join(' '));
	log.info('    Current CR: ' + event.objectId);
}

if (checkArguments(0, [], []))
{
	// Get own status
	// crstatus is never null
	const crstate_attr_name = 'crstatus';
	const original_state = change.collectAttributes(event.triggerUser.token, event, [crstate_attr_name]).get(crstate_attr_name).toString();
	
	if (script_debug)
	{
		log.info(log_header + 'Original CR has status \'' + original_state + '\'');
	}

	// Get to state
	const prefix = 'duplicate_';
	const to_state = prefix + original_state; 
	
	if (script_debug)
	{
		log.info(log_header + 'Target status is \'' + to_state + '\'');
	}

	// Get duplicate CRs
	const relationship = 'duplicate';
	var attribute_names = [crstate_attr_name];
	
	if (script_dcm_modifiable_check)
	{
		attribute_names.push('modifiable_in');
	}
	
	var items = change.getRelatedFrom(event.triggerUser.token, event.objectId, relationship, attribute_names);

	// Filter list
	if (script_debug)
	{
		log.info(log_header + 'Item list: ' + items.join(', '));
	}

	var exlude_states = ['duplicate_evaluated', 'duplicate_concluded']
	items = filterItems(items, exlude_states);
	
	if (script_debug) 
	{
		log.info(log_header + 'Filtered item list: ' + items.join(', '));
	}

	// Handle each duplicate CR
	for (var i = 0; i < items.length; i++)
	{
		transitionCR(items[i], to_state);
	}
}


/**
 * Checks the script arguments:
 * - Check argument count
 * - Check that all specified options are valid
 * - Check that all required options are specified
 *
 * Returns: true, if succesfull. false otherwise
 */
function checkArguments(arg_count, allowed_options, required_options)
{
	var message;

	// Check argument count
	if (options.args.length < arg_count)
	{
		message = 'Insufficient arguments to run the script (' + options.args.length +' arguments)';
	}

	// Check that all specified options are valid
	if (!message)
	{
		var all_options = options.getOptions();
		var i = all_options.keySet().iterator();
		
		while (!message && i.hasNext())
		{
			var option = i.next();
			var is_valid = false;
			
			for (var j = 0; j < allowed_options.length; j++)
			{
				if (option == allowed_options[j])
				{
					is_valid = true;
					break;
				}
			}

			if (!is_valid)
			{
				message = 'Invalid option \'' + option + '\'';
			}
		}
	}

	// Check that all required options are specified
	if (!message) 
	{
		for (var i=0; i < required_options.length; i++)
		{
			var option = options.getOption(required_options[i]);
			
			if (!option)
			{
				message = 'Missing option \'' + required_options[i] + '\'';
			}
		}
	}

	if (message)
	{
		log.error(log_header);
		log.error('   ' + message);
		log.error('   Usage: ' + script_usage);
		log.error('   Example: ' + script_example);
		return false;
	}

	return true;
}

/**
 * Checks, if an attribute has no value set.
 *
 * The function is necessary, because Attribute.toString() returns 'null' instead of null. By
 * returning 'null' it cannot be distinguished between an without a value and one with the 
 * value 'null'.
 *
 * Returns: True, if the attriute has no value set.
 */
function isAttributeNull(attribute)
{
	if (((!attribute) || (attribute.type == Attribute.Type.STRING) && (attribute.getStringValue() == null))
		|| ((attribute.type == Attribute.Type.DATE) && (attribute.getDateValue() == null))
		|| ((attribute.type == Attribute.Type.BOOLEAN) && (attribute.getBooleanValue() == null)))
	{
		return true;
	}

	return false;
}


/**
 * Filters duplicate CRs from the list of items
 * - The state is in the list of states to be excluded
 * - The item is not modifiable in the current database and script_dcm_modifiable_check is true
 *
 * Returns: -
 */
function filterItems(items, exclude_states)
{
	var new_items = new Array();
	
	for (var i = 0; i < items.length; i++)
	{
		// The state is in the list of states to be excluded
		const item_status = items[i].getAttribute('crstatus').toString();
		var is_excluded = false;
	
		for (var j = 0; j < exclude_states.length; j++)
		{
			if (exclude_states[j].equals(item_status)) // == is always false
			{
				is_excluded = true;
				break;
			}
		}
		
		if (is_excluded)
		{
			if (script_debug) 
			{
				log.info(log_header + 'Skipping exluded state \'' + item_status + '\' of item ' + items[i]);
			}

			continue;
		}

		// The item is not modifiable in the current database and script_dcm_modifiable_check is true
		// modifiable_in is never null, if DCM is enabled
		if (script_dcm_modifiable_check &&
			change.isDcmEnabled(event.triggerUser.token) &&
			!items[i].getAttribute('modifiable_in').toString().equals(change.getDbid(event.triggerUser.token))) // == is always false
		{
			if (script_debug) 
			{
				log.info(log_header + 'Skipping non modifiable item. Item \'' + items [i]+ 
				'\' is not modifiable in database \'' + change.getDbid(event.triggerUser.token) + '\'');
			}

			continue;
		}

		new_items.push(items[i]);
	}

	return new_items;
}


/**
 * Transitions the CR to the target state
 *
 */
function transitionCR(item, to_state)
{
	var attributes = [];

	try 
	{
		if (script_debug)
		{
			log.info(log_header + 'Transitioning CR ' + item.crid);
		}

		change.transition(event.triggerUser.token, item.crid, to_state, []);
	}
	catch (error)
	{
		const from_state = items.getAttribute('crstatus').toString();
		var message = change.getMessage('errorTransitioningCRFromXtoStateYerrorZ', [from_state, to_state, error.message || error.toString()]);
		log.error(log_header + message);
		event.fail(message);
	}
}
